import pandas as pd
import numpy as np
from openai.embeddings_utils import get_embedding, cosine_similarity
import yaml
import os

params = yaml.safe_load(open("params.yaml"))
# api_key = params["get_embeddings"]["api_token"]
# api_key = os.environ.get("API_KEY")

api_key = os.environ['OPENAI_API_KEY']


def import_csv(file_path, use_cols=None):
    """
    loads a csv file into memory
    :input:
        file_path: str, directory path of the csv file to be imported
        use_cols: list, collection of the column headers to be imported
    :output:
        df: pandas DataFrame, dataframe of the loaded csv file
    """
    df = pd.read_csv(file_path, usecols=use_cols, low_memory=False)
    print("File has", df.shape[0], "rows")

    return df


def save_csv_file(df, destination_path):
    """
    save data to csv file without index
    :input:
        df: pandas DataFrame, finalised data to be saved to disk
        destination_path: str, file directory where the data should be saved to
    :output:
        Nil
    """
    df.to_csv(destination_path, index=False)
    print("Done saving csv file")

    return


def search_courses(df, query, n=5, threshold=0.7, pprint=False):
    """
    search through course data and find and rank search results based on cosine similarity between query and course content
    :input:
        df: pandas DataFrame, input dataframe with content embeddings
        query: str, search query input by user
        n: int, number of top ranked results to display
        threshold: float, cut-off similarity score for search results
        pprint: True/False, determines if results are printed out in console
    :output:
        final_json: json, json formatted search results
    """
    course_embedding = get_embedding(
        query,
        engine="text-embedding-ada-002"
    )
    df["similarity"] = df.embedding.apply(lambda x: cosine_similarity(x, course_embedding))

    results = (
        df.sort_values("similarity", ascending=False).head(n)
    )

    results = results.query(f'similarity > {threshold}')
    
    output = {}
    _list = []
    keys = range(len(results))
    
    for title, obj, aot, skill, sim in zip(
        results['Course Title'], results['Course Objective'], 
        results["course_skill_list"], results['Area Of Training'], 
        results.similarity):
        results_dic = {
            "course_title": title,
            "course_objective": obj,
            "course_skills": skill,
            "area_of_training": aot,
            "similarity_score": np.round(sim, 2)
        }
        _list.append(results_dic)
    
    for i in keys:
        output[i] = _list[i]
    if len(output) == 0:
        final_json = {}
    else:
        final_json = {
            "search_results": output
        }

    if pprint:
        i = 1
        for r in results_final['Course Title']:
            print(f"{i}. Course Title: {r[:100]}")
            print()
            i += 1

    return final_json



import yaml
import os
from botocore.exceptions import NoCredentialsError
import boto3
import openai
from cachetools import cached

from course_search.utils import *

params = yaml.safe_load(open("params.yaml"))
index_file = params["merge_indices"]["indexed_file_path"]
s3_file_name = params["download"]["s3_file_name"]
bucket_name = params["download"]["bucket_name"]

api_key = os.environ.get("OPENAI_API_KEY")
aws_access_key_id = os.environ.get("AWS_ACCESS_KEY_ID")
aws_secret_access_key = os.environ.get("AWS_SECRET_ACCESS_KEY")

os.environ['OPENAI_API_KEY'] = api_key

openai.api_key = api_key

def download_file(bucket_name, file_key, local_dir):
    s3 = boto3.resource("s3", 
                        aws_access_key_id=aws_access_key_id, 
                        aws_secret_access_key=aws_secret_access_key)
    bucket = s3.Bucket(bucket_name)
    bucket.download_file(file_key, local_dir)

@cached(cache={})
def load_index(index_file):
    catalog_df = import_csv(file_path=index_file)
    catalog_df['embedding'] = catalog_df.embedding.apply(eval).apply(np.array)
    return catalog_df

def file_check(index_file):
    if not os.path.exists(index_file):
        download_file(bucket_name=bucket_name, file_key=s3_file_name, local_dir=index_file)
    
def search_for_courses(query, top_n=5, threshold=0.81, index_file=index_file):
    file_check(index_file)
    
    # Load in the course catalog with embeddings
    catalog_df = load_index(index_file=index_file)
    print("file loaded")

    # call util function search_courses to return top matches in json
    result_json = search_courses(df=catalog_df, query=query.strip(), n=top_n, threshold=threshold, pprint=False)
    
    return result_json

# if __name__ == "__main__":
#     res = search_for_courses(query="pottery")
#     print(res)


from pathlib import Path
import course_search

# Project Directories
PACKAGE_ROOT = Path(course_search.__file__).resolve().parent
ROOT = PACKAGE_ROOT.parent
CONFIG_FILE_PATH = PACKAGE_ROOT / "params.yaml"
DATASET_DIR = PACKAGE_ROOT / "index"
PICKLE_FILE_PATH = PACKAGE_ROOT / "index/ssg_courses_full_indexed.csv"


def find_config_file() -> Path:
    """Locate the configuration file."""
    if CONFIG_FILE_PATH.is_file():
        return CONFIG_FILE_PATH
    raise Exception(f"Config not found at {CONFIG_FILE_PATH!r}")


from course_search.course_search import search_for_courses

test_query = " I want to learn pottery "

def test_course_search(query=test_query):
    course_search_result = search_for_courses(test_query)

    assert isinstance(course_search_result, dict)
    assert len(course_search_result) == 1